import java.util.HashMap;

/**
The contents of the table cells in the target row of a rule.

@author Michael Breen
*/
class TargetRow
{
   /**
   Maps each variable to its new value, or to a set of
   values to any of which it is allowed to change.
   Unmapped variables are "no change".
   */
   private HashMap valueSets = new HashMap();

   /**
   Creates a target row in which all variables are
   "no change".
   */
   TargetRow() { }

   /**
   @param values the new value of a variable or a set of
   possible new values
   @return this object (to facilitate invocation chaining)
   @throws IllegalArgumentException if the argument is null
   */
   TargetRow setValues(ValueSet values)
   {
      if (values == null)
         throw new IllegalArgumentException();
      this.valueSets.put(values.getVariable(), values);
      return this;
   }

   /**
   This returns the contents of the target row with any
   "no change" elements being replaced by the value of
   the corresponding variable in the argument.
   @param current a combination defining the current state
   @return a mask defining the next states to which this
   target row allows transitions from the argument
   */
   CombinationMask transform(Combination current)
   {
      CombinationMask targets = new CombinationMask(current);
      targets.valueSets.putAll(this.valueSets);
      return targets;
   }
}
