import java.util.ArrayList;

/**
A variable defined in the model.

@author Michael Breen
*/
public final class Variable
{
   /** A list of all the variables. */
   private static ArrayList varList = new ArrayList();

<#list usable_variables as var>
   <#if var.description?size != 0>
      <@prologue_comment var.description/>
   </#if>
   public static final Variable ${variable_id(var.@name)} =
      new Variable("${var.@name}");
</#list>

   /** This variable's identifier as defined in the model. */
   private String id;

   /** The possible values of this variable. */
   private ArrayList values = new ArrayList();

   /** The set of all values of this variable. */
   private ValueSet allVals = null;

   /** This variable's index in the list of variables. */
   private int listIndex;

   /**
   Get the number of variables in the model.
   @return the total number of variables
   */
   public static int numVars() { return Variable.varList.size(); }

   /**
   Get a variable from its index in the list of
   variables.
   @param index a number in the range 0 ..
   ({@link #numVars()} - 1)
   @return the variable at the specified index
   */
   public static Variable getVariable(int index)
   {
      return (Variable) Variable.varList.get(index);
   }

   /** @param name the new variable's identifier */
   private Variable(String name)
   {
      this.id = name;
      this.listIndex = Variable.varList.size();
      Variable.varList.add(this);
   }

   /**
   The string representation of a variable is simply
   its name in the model.
   @return this variable's identifier in the model
   */
   public String toString() { return this.id; }

   /**
   Get the index of this variable in the list of
   all variables.
   @return a number in the range 0 ..
   ({@link #numVars()} - 1)
   */
   public int getIndex() { return this.listIndex; }

   /**
   Get the number of possible values of this variable.
   @return the number of values of this variable
   */
   public int numVals() { return this.values.size(); }

   /**
   Get the value at a particular index in this
   variable's list of values.
   @param index a number in the range 0 ..
   ({@link #numVals()} - 1)
   @return the value at the specified index
   */
   public Value getValue(int index)
   {
      return (Value) this.values.get(index);
   }

   /**
   Get the set of all possible values of this variable.
   @return the "don't care" set for this variable
   */
   public ValueSet dontCare()
   {
      if (this.allVals == null)
      {
         this.allVals = getValue(0);
         for (int i = 1; i < numVals(); ++i)
            this.allVals = this.allVals.or(getValue(i));
      }
      return this.allVals;
   }

   /**
   @param value a possible value of this variable
   @return the index of the new value in the list of values
   */
   int addValue(Value value)
   {
      this.values.add(value);
      return numVals() - 1;
   }

   /*
   For each variable, force initialization of the class
   of which the variable's values are instances.
   This ensures that (for example) numVals() isn't called
   before the values of the variable have been created.
   */
   static
   {
      try
      {
<#list usable_variables as var>
         Class.forName("${package_prefix()
            }${variable_id(var.@name)}");
</#list>
      }
      catch (ClassNotFoundException e)
      {
      }
   }
}
